using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;


namespace VolleyballGame
{
    public class Ball : DrawableGameComponent
    {


        private static readonly Vector2 gravity = new Vector2(0, 0.3f),
                                        maxSpeed = new Vector2(16, 18),
                                        minSpeed = new Vector2(0.5f, 0.2f),
                                        speedFall = new Vector2(0.999f, 1),
                                        positionOffset = new Vector2(35, 35);

        private const float radius = 35;

        private Texture2D texture;
        private Rectangle bounds, net;
        private Vector2 speed = Vector2.Zero;
        private Vector2 position;

        private bool netCollision;


        public Ball(Rectangle bounds, Vector2 position, Rectangle net, Game game)
            : base(game)
        {
            this.bounds = bounds;
            this.position = position;
            this.net = net;
        }

        protected override void LoadContent()
        {
            texture = Game.Content.Load<Texture2D>("ball");
            base.LoadContent();
        }

        public override void Update(GameTime gameTime)
        {
            HandleNetCollision();

            if (position.Y + texture.Height > bounds.Bottom)
            {
                Bounce(Vector2.UnitY, 0.5f, Vector2.Zero);
                position.Y = bounds.Bottom - texture.Height;
            }
            else if (position.Y < bounds.Top)
            {
                Bounce(Vector2.UnitY, 0.8f, Vector2.Zero);
                position.Y = bounds.Top;
            }

            if (position.X + texture.Width > bounds.Right)
            {
                Bounce(Vector2.UnitX, 0.8f, Vector2.Zero);
                position.X = bounds.Right - texture.Width;
            }
            else if (position.X < bounds.Left)
            {
                Bounce(Vector2.UnitX, 0.8f, Vector2.Zero);
                position.X = bounds.Left;
            }


            if (position.Y + texture.Height < bounds.Bottom - 5)
                speed += gravity;

            speed *= speedFall;

            if (Math.Abs(speed.X) > maxSpeed.X)
                speed.X = ((speed.X > 0) ? 1 : -1) * maxSpeed.X;
            else if (Math.Abs(speed.X) < minSpeed.X)
                speed.X = 0;

            if (Math.Abs(speed.Y) > maxSpeed.Y)
                speed.Y = ((speed.Y > 0) ? 1 : -1) * maxSpeed.Y;
            else if (Math.Abs(speed.Y) < minSpeed.Y)
                speed.Y = 0;


            position += speed;

            base.Update(gameTime);
        }


        public override void Draw(GameTime gameTime)
        {
            SpriteBatch spriteBatch = (SpriteBatch)Game.Services.GetService(typeof(SpriteBatch));
            spriteBatch.Draw(texture, position, Color.White);

            base.Draw(gameTime);
        }

        protected override void UnloadContent()
        {
            texture.Dispose();
            base.UnloadContent();
        }

        public void Bounce(Vector2 normal, float speedChange, Vector2 addSpeed)
        {
            speed = speedChange * Vector2.Reflect(speed, normal);
            speed += addSpeed;
        }

        private void HandleNetCollision()
        {
            if (Center.X + Radius > net.Left && Center.X - Radius < net.Right)
            {
                if (Center.Y > net.Top + net.Width / 2)
                {
                    Bounce(Vector2.UnitX, 0.5f, Vector2.Zero);
                    if (Center.X < net.Left)
                        position.X = net.Left - radius - positionOffset.X;
                    else
                        position.X = net.Right + radius - positionOffset.X;

                    netCollision = false;
                }
                else
                {
                    Vector2 diff = Center
                        - new Vector2(net.Center.X, net.Top + net.Width / 2);

                    if (diff.Length() < net.Width / 2 + Radius)
                    {
                        if (!netCollision)
                            Bounce(diff, 0.3f, Vector2.Zero);
                        netCollision = true;
                    }
                    else
                        netCollision = false;
                }
            }
            else
                netCollision = false;
        }

        public Vector2 Center
        {
            get { return position + positionOffset; }
        }

        public float Radius
        {
            get { return radius; }
        }

    }
}